//-----------------------------------------------------------------------------
// GA2K
//
// ISDL, Doshisha University Distributed Genetic Algorithm Software
// Version 1.2
//-----------------------------------------------------------------------------
// Parallel Genetic Algorithm Team of Intelligent Systems Design Laborato
// ry, Doshisha University.
//
// Version 0.1: 1999/05/05
// Copyright (C) 1999 Jun-ichi Yoshida and Ikki Ohmukai, All rights reserved.
// Version 1.0: 2001/12/13
// Copyright (C) 2001 Jun-ichi Yoshida, Tomoyuki Hiroyasu and Mitsunori Miki,
// All rights reserved.
// Version 1.1: 2002/01/17
// Copyright (C) 2002 Jun-ichi Yoshida, Masaki Sano, Tomoyuki Hiroyasu and Mitsunori Miki, All rights reserved.
// Version 1.2: 2002/06/04
// Copyright (C) 2002 Jun-ichi Yoshida, Masaki Sano, Tomoyuki Hiroyasu and Mitsunori Miki, All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notce and this list of conditions.
// 2. Redistributions in binary form must reproduce the above  copyright
//    notice and this list of conditions in the documentation and/or other
//    materials provided with the distribution.
// 3. The names of copyrighters of this software  should not be used to
//    endorse or promote products derived from this software without specific
//    prior written permission.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// ga2k_mpi.h
//-----------------------------------------------------------------------------

#ifndef _GA2K_MPI_H_
#define _GA2K_MPI_H_

#include <vector>

#include "ga2k.h"
#include <mpi.h>

class Island;
class Gene;


class MPIGlobe
{
  public:
	//data
	Island			island;
	Island			island_0;

	int					generation;
	int					maxGeneration;
	int					convGeneration;


	int					migrationInterval;
	int					numIsland;
	int					populationSize;
	int					totalPopulationSize;
	int					migrantSize;
	int					nativeSize;

	int 				fInitialPopulation;

	double				maxFitness;
	int					maxTrial;
	double				sumMaxFitness;
	int					sumGeneration;
	int					numConvergent;
	int					totalEvaluate;
	int					sumTotalEvaluate;

	vector<double>		sumFitness;

	Gene				elite;
	bool				fStop;
	bool				fConv;
	bool				fTotalPopSize;
	bool				fOutput;

	// data (for mpi)
	int					root;
	int 				nProcs;
	int					myRank;
	char 				hostname[MPI_MAX_PROCESSOR_NAME];
	RandomRing	destination;


	//method (constructor)
	MPIGlobe(int* pargc, char*** pargv);
	~MPIGlobe();
	void initialize();
	void initialize(int argc, char *argv[]);
	void establish();

	//method (operator)
	void emigrate();
	void execute();
	void showParameters();
	void finalize(bool b);

	//method (for mpi)
	void synchronize() const;
	void setMaxFitness();


  private:
	MPIGlobe() : destination(1) {}
	MPIGlobe(const MPIGlobe& ob) : destination(1) {}
	MPIGlobe& operator=(const MPIGlobe& ob) {}
};

#endif // _GA2K_MPI_H_